library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity axi_memiface_32_bram_ext_ctrl is
  port (
    clk : in std_logic;
    reset : in std_logic;
    data_address : in std_logic_vector(32-1 downto 0);
    data_din : in std_logic_vector(32-1 downto 0);
    data_dout : out std_logic_vector(32-1 downto 0);
    data_we : in std_logic;
    data_oe : in std_logic;
    data_address_external : in std_logic_vector(32-1 downto 0);
    data_din_external : in std_logic_vector(32-1 downto 0);
    data_dout_external : out std_logic_vector(32-1 downto 0);
    data_we_external : in std_logic;
    data_oe_external : in std_logic;
    ext_ctrl : in std_logic;
    ext_ctrl_flag : out std_logic;
    data_length : out std_logic_vector(32-1 downto 0);
    busy : out std_logic;
    write_kick : in std_logic;
    read_kick : in std_logic;
    burst_size : in std_logic_vector(32-1 downto 0);
    axi_addr : in std_logic_vector(32-1 downto 0);
    forbid : in std_logic;
    axi_reader_ARADDR : out std_logic_vector(32-1 downto 0);
    axi_reader_ARLEN : out std_logic_vector(8-1 downto 0);
    axi_reader_ARVALID : out std_logic;
    axi_reader_ARREADY : in std_logic;
    axi_reader_ARSIZE : out std_logic_vector(3-1 downto 0);
    axi_reader_ARBURST : out std_logic_vector(2-1 downto 0);
    axi_reader_ARCACHE : out std_logic_vector(4-1 downto 0);
    axi_reader_ARPROT : out std_logic_vector(3-1 downto 0);
    axi_reader_RDATA : in std_logic_vector(32-1 downto 0);
    axi_reader_RRESP : in std_logic_vector(2-1 downto 0);
    axi_reader_RLAST : in std_logic;
    axi_reader_RVALID : in std_logic;
    axi_reader_RREADY : out std_logic;
    axi_writer_AWADDR : out std_logic_vector(32-1 downto 0);
    axi_writer_AWLEN : out std_logic_vector(8-1 downto 0);
    axi_writer_AWVALID : out std_logic;
    axi_writer_AWSIZE : out std_logic_vector(3-1 downto 0);
    axi_writer_AWBURST : out std_logic_vector(2-1 downto 0);
    axi_writer_AWCACHE : out std_logic_vector(4-1 downto 0);
    axi_writer_AWPROT : out std_logic_vector(3-1 downto 0);
    axi_writer_AWREADY : in std_logic;
    axi_writer_WDATA : out std_logic_vector(32-1 downto 0);
    axi_writer_WLAST : out std_logic;
    axi_writer_WVALID : out std_logic;
    axi_writer_WREADY : in std_logic;
    axi_writer_WSTRB : out std_logic_vector(4-1 downto 0);
    axi_writer_BRESP : in std_logic_vector(2-1 downto 0);
    axi_writer_BVALID : in std_logic;
    axi_writer_BREADY : out std_logic
  );
end axi_memiface_32_bram_ext_ctrl;

architecture RTL of axi_memiface_32_bram_ext_ctrl is

  attribute mark_debug : string;
  attribute keep : string;

  component dualportram
    generic (
      WIDTH : integer := 32;
      DEPTH : integer := 10;
      WORDS : integer := 1024
    );
    port (
      clk : in std_logic;
      reset : in std_logic;
      length : out signed(31 downto 0);
      address : in signed(31 downto 0);
      din : in signed(WIDTH-1 downto 0);
      dout : out signed(WIDTH-1 downto 0);
      we : in std_logic;
      oe : in std_logic;
      address_b : in signed(31 downto 0);
      din_b : in signed(WIDTH-1 downto 0);
      dout_b : out signed(WIDTH-1 downto 0);
      we_b : in std_logic;
      oe_b : in std_logic
    );
  end component dualportram;
  component simple_fifo_32
    port (
      clk : in std_logic;
      reset : in std_logic;
      we : in std_logic;
      din : in std_logic_vector(32-1 downto 0);
      re : in std_logic;
      dout : out std_logic_vector(32-1 downto 0);
      empty : out std_logic;
      full : out std_logic;
      count : out std_logic_vector(32-1 downto 0)
    );
  end component simple_fifo_32;

  signal clk_sig : std_logic;
  signal reset_sig : std_logic;
  signal data_address_sig : std_logic_vector(32-1 downto 0);
  signal data_din_sig : std_logic_vector(32-1 downto 0);
  signal data_dout_sig : std_logic_vector(32-1 downto 0);
  signal data_we_sig : std_logic;
  signal data_oe_sig : std_logic;
  signal data_address_external_sig : std_logic_vector(32-1 downto 0);
  signal data_din_external_sig : std_logic_vector(32-1 downto 0);
  signal data_dout_external_sig : std_logic_vector(32-1 downto 0);
  signal data_we_external_sig : std_logic;
  signal data_oe_external_sig : std_logic;
  signal ext_ctrl_sig : std_logic;
  signal ext_ctrl_flag_sig : std_logic;
  signal data_length_sig : std_logic_vector(32-1 downto 0);
  signal busy_sig : std_logic;
  signal write_kick_sig : std_logic;
  signal read_kick_sig : std_logic;
  signal burst_size_sig : std_logic_vector(32-1 downto 0);
  signal axi_addr_sig : std_logic_vector(32-1 downto 0);
  signal forbid_sig : std_logic;
  signal axi_reader_ARADDR_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal axi_reader_ARLEN_sig : std_logic_vector(8-1 downto 0) := (others => '0');
  signal axi_reader_ARVALID_sig : std_logic := '0';
  signal axi_reader_ARREADY_sig : std_logic;
  signal axi_reader_ARSIZE_sig : std_logic_vector(3-1 downto 0);
  signal axi_reader_ARBURST_sig : std_logic_vector(2-1 downto 0);
  signal axi_reader_ARCACHE_sig : std_logic_vector(4-1 downto 0);
  signal axi_reader_ARPROT_sig : std_logic_vector(3-1 downto 0);
  signal axi_reader_RDATA_sig : std_logic_vector(32-1 downto 0);
  signal axi_reader_RRESP_sig : std_logic_vector(2-1 downto 0);
  signal axi_reader_RLAST_sig : std_logic;
  signal axi_reader_RVALID_sig : std_logic;
  signal axi_reader_RREADY_sig : std_logic := '0';
  signal axi_writer_AWADDR_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal axi_writer_AWLEN_sig : std_logic_vector(8-1 downto 0) := (others => '0');
  signal axi_writer_AWVALID_sig : std_logic := '0';
  signal axi_writer_AWSIZE_sig : std_logic_vector(3-1 downto 0);
  signal axi_writer_AWBURST_sig : std_logic_vector(2-1 downto 0);
  signal axi_writer_AWCACHE_sig : std_logic_vector(4-1 downto 0);
  signal axi_writer_AWPROT_sig : std_logic_vector(3-1 downto 0);
  signal axi_writer_AWREADY_sig : std_logic;
  signal axi_writer_WDATA_sig : std_logic_vector(32-1 downto 0);
  signal axi_writer_WLAST_sig : std_logic := '0';
  signal axi_writer_WVALID_sig : std_logic := '0';
  signal axi_writer_WREADY_sig : std_logic;
  signal axi_writer_WSTRB_sig : std_logic_vector(4-1 downto 0);
  signal axi_writer_BRESP_sig : std_logic_vector(2-1 downto 0);
  signal axi_writer_BVALID_sig : std_logic;
  signal axi_writer_BREADY_sig : std_logic := '0';

  signal data_address_act : signed(32-1 downto 0);
  signal data_din_act : signed(32-1 downto 0);
  signal data_dout_act : signed(32-1 downto 0);
  signal data_we_act : std_logic;
  signal data_oe_act : std_logic;
  signal tmp_0001 : std_logic_vector(32-1 downto 0);
  signal tmp_0002 : std_logic_vector(32-1 downto 0);
  signal tmp_0003 : std_logic;
  signal tmp_0004 : std_logic;
  signal BUF_clk : std_logic;
  signal BUF_reset : std_logic;
  signal BUF_length : signed(32-1 downto 0);
  signal BUF_address : signed(32-1 downto 0);
  signal BUF_din : signed(32-1 downto 0);
  signal BUF_dout : signed(32-1 downto 0);
  signal BUF_we : std_logic;
  signal BUF_oe : std_logic;
  signal BUF_address_b : signed(32-1 downto 0) := (others => '0');
  signal BUF_din_b : signed(32-1 downto 0) := (others => '0');
  signal BUF_dout_b : signed(32-1 downto 0);
  signal BUF_we_b : std_logic := '0';
  signal BUF_oe_b : std_logic := '0';
  signal read_state_busy : std_logic := '0';
  signal write_state_busy : std_logic := '0';
  signal tmp_0005 : std_logic;
  signal tmp_0006 : std_logic;
  signal hdl_write_kick_d : std_logic := '0';
  signal tmp_0007 : std_logic;
  signal tmp_0008 : std_logic;
  signal hdl_read_kick_d : std_logic := '0';
  signal tmp_0009 : std_logic;
  signal tmp_0010 : std_logic;
  signal tmp_0011 : std_logic;
  signal tmp_0012 : std_logic;
  type Type_axi is (
    axi_IDLE,
    axi_sjr_tmp_state_0000,
    axi_sjr_tmp_state_0001,
    axi_sjr_tmp_state_0002,
    axi_sjr_tmp_state_0003,
    axi_sjr_tmp_state_0004,
    axi_sjr_tmp_state_0005,
    axi_sjr_tmp_state_0006  
  );
  signal axi : Type_axi := axi_IDLE;
  signal axi_delay : signed(32-1 downto 0) := (others => '0');
  signal sjr_generate_U_0013_clk : std_logic;
  signal sjr_generate_U_0013_reset : std_logic;
  signal sjr_generate_U_0013_we : std_logic;
  signal sjr_generate_U_0013_din : std_logic_vector(32-1 downto 0);
  signal sjr_generate_U_0013_re : std_logic;
  signal sjr_generate_U_0013_dout : std_logic_vector(32-1 downto 0);
  signal sjr_generate_U_0013_empty : std_logic;
  signal sjr_generate_U_0013_full : std_logic;
  signal sjr_generate_U_0013_count : std_logic_vector(32-1 downto 0);
  signal tmp_0014 : std_logic := '0';
  signal tmp_0015 : std_logic;
  signal tmp_0016 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0017 : std_logic_vector(32-1 downto 0);
  signal tmp_0018 : std_logic;
  signal tmp_0019 : std_logic;
  signal tmp_0020 : std_logic_vector(32-1 downto 0);
  signal tmp_0021 : std_logic_vector(32-1 downto 0) := (others => '0');
  signal tmp_0022 : std_logic;
  signal tmp_0023 : std_logic_vector(32-1 downto 0);
  signal tmp_0024 : std_logic_vector(32-1 downto 0);
  signal tmp_0025 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0026 : signed(32-1 downto 0);
  signal tmp_0027 : signed(32-1 downto 0);
  signal tmp_0028 : std_logic := '0';
  signal tmp_0029 : std_logic;
  signal tmp_0030 : std_logic;
  signal tmp_0031 : std_logic_vector(32-1 downto 0);
  signal tmp_0032 : std_logic_vector(8-1 downto 0);
  signal tmp_0033 : std_logic;
  signal tmp_0034 : signed(32-1 downto 0);
  signal tmp_0035 : std_logic;
  signal tmp_0036 : std_logic;
  signal tmp_0037 : std_logic;
  signal tmp_0038 : std_logic;
  signal tmp_0039 : std_logic;
  signal tmp_0040 : std_logic;
  signal tmp_0041 : std_logic;
  signal tmp_0042 : std_logic;
  signal tmp_0043 : std_logic;
  signal tmp_0044 : std_logic;
  signal tmp_0045 : std_logic;
  signal tmp_0046 : std_logic;
  signal tmp_0047 : std_logic;
  signal tmp_0048 : std_logic;
  signal tmp_0049 : std_logic;
  signal tmp_0050 : std_logic;
  signal tmp_0051 : std_logic;
  signal tmp_0052 : std_logic;
  signal tmp_0053 : std_logic;
  signal tmp_0054 : std_logic;
  signal tmp_0055 : std_logic;
  signal tmp_0056 : std_logic;
  signal tmp_0057 : std_logic;
  signal tmp_0058 : signed(32-1 downto 0);
  signal tmp_0059 : std_logic;
  signal tmp_0060 : std_logic;
  signal tmp_0061 : std_logic;
  signal tmp_0062 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0063 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0064 : std_logic_vector(32-1 downto 0);
  signal tmp_0065 : std_logic_vector(8-1 downto 0);
  signal tmp_0066 : std_logic;
  signal tmp_0067 : std_logic;
  signal tmp_0068 : signed(32-1 downto 0);
  signal tmp_0069 : signed(32-1 downto 0);
  signal tmp_0070 : signed(32-1 downto 0);
  signal tmp_0071 : signed(32-1 downto 0);
  signal tmp_0072 : std_logic;

begin

  clk_sig <= clk;
  reset_sig <= reset;
  data_address_sig <= data_address;
  data_din_sig <= data_din;
  data_dout <= data_dout_sig;
  data_dout_sig <= std_logic_vector(data_dout_act);

  data_we_sig <= data_we;
  data_oe_sig <= data_oe;
  data_address_external_sig <= data_address_external;
  data_din_external_sig <= data_din_external;
  data_dout_external <= data_dout_external_sig;
  data_dout_external_sig <= std_logic_vector(data_dout_act);

  data_we_external_sig <= data_we_external;
  data_oe_external_sig <= data_oe_external;
  ext_ctrl_sig <= ext_ctrl;
  ext_ctrl_flag <= ext_ctrl_flag_sig;
  ext_ctrl_flag_sig <= ext_ctrl_sig;

  data_length <= data_length_sig;
  data_length_sig <= std_logic_vector(BUF_length);

  busy <= busy_sig;
  busy_sig <= tmp_0006;

  write_kick_sig <= write_kick;
  read_kick_sig <= read_kick;
  burst_size_sig <= burst_size;
  axi_addr_sig <= axi_addr;
  forbid_sig <= forbid;
  axi_reader_ARADDR <= axi_reader_ARADDR_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_reader_ARADDR_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_reader_ARADDR_sig <= axi_addr_sig;
        end if;
      end if;
    end if;
  end process;

  axi_reader_ARLEN <= axi_reader_ARLEN_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_reader_ARLEN_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_reader_ARLEN_sig <= tmp_0065;
        end if;
      end if;
    end if;
  end process;

  axi_reader_ARVALID <= axi_reader_ARVALID_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_reader_ARVALID_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_reader_ARVALID_sig <= tmp_0012;
        elsif axi = axi_sjr_tmp_state_0005 then
          axi_reader_ARVALID_sig <= tmp_0066;
        end if;
      end if;
    end if;
  end process;

  axi_reader_ARREADY_sig <= axi_reader_ARREADY;
  axi_reader_ARSIZE <= axi_reader_ARSIZE_sig;
  axi_reader_ARSIZE_sig <= "010";

  axi_reader_ARBURST <= axi_reader_ARBURST_sig;
  axi_reader_ARBURST_sig <= "01";

  axi_reader_ARCACHE <= axi_reader_ARCACHE_sig;
  axi_reader_ARCACHE_sig <= X"3";

  axi_reader_ARPROT <= axi_reader_ARPROT_sig;
  axi_reader_ARPROT_sig <= "000";

  axi_reader_RDATA_sig <= axi_reader_RDATA;
  axi_reader_RRESP_sig <= axi_reader_RRESP;
  axi_reader_RLAST_sig <= axi_reader_RLAST;
  axi_reader_RVALID_sig <= axi_reader_RVALID;
  axi_reader_RREADY <= axi_reader_RREADY_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_reader_RREADY_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_reader_RREADY_sig <= '1';
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWADDR <= axi_writer_AWADDR_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_writer_AWADDR_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_writer_AWADDR_sig <= axi_addr_sig;
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWLEN <= axi_writer_AWLEN_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_writer_AWLEN_sig <= (others => '0');
      else
        if axi = axi_IDLE then
          axi_writer_AWLEN_sig <= tmp_0032;
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWVALID <= axi_writer_AWVALID_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_writer_AWVALID_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_AWVALID_sig <= tmp_0008;
        elsif axi = axi_sjr_tmp_state_0000 then
          axi_writer_AWVALID_sig <= tmp_0033;
        end if;
      end if;
    end if;
  end process;

  axi_writer_AWSIZE <= axi_writer_AWSIZE_sig;
  axi_writer_AWSIZE_sig <= "010";

  axi_writer_AWBURST <= axi_writer_AWBURST_sig;
  axi_writer_AWBURST_sig <= "01";

  axi_writer_AWCACHE <= axi_writer_AWCACHE_sig;
  axi_writer_AWCACHE_sig <= X"3";

  axi_writer_AWPROT <= axi_writer_AWPROT_sig;
  axi_writer_AWPROT_sig <= "000";

  axi_writer_AWREADY_sig <= axi_writer_AWREADY;
  axi_writer_WDATA <= axi_writer_WDATA_sig;
  axi_writer_WDATA_sig <= tmp_0017;

  axi_writer_WLAST <= axi_writer_WLAST_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_writer_WLAST_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_WLAST_sig <= '0';
        elsif axi = axi_sjr_tmp_state_0003 then
          axi_writer_WLAST_sig <= tmp_0048;
        elsif axi = axi_sjr_tmp_state_0004 then
          axi_writer_WLAST_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  axi_writer_WVALID <= axi_writer_WVALID_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_writer_WVALID_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_WVALID_sig <= '0';
        elsif axi = axi_sjr_tmp_state_0003 then
          axi_writer_WVALID_sig <= tmp_0054;
        elsif axi = axi_sjr_tmp_state_0004 then
          axi_writer_WVALID_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  axi_writer_WREADY_sig <= axi_writer_WREADY;
  axi_writer_WSTRB <= axi_writer_WSTRB_sig;
  axi_writer_WSTRB_sig <= X"f";

  axi_writer_BRESP_sig <= axi_writer_BRESP;
  axi_writer_BVALID_sig <= axi_writer_BVALID;
  axi_writer_BREADY <= axi_writer_BREADY_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi_writer_BREADY_sig <= '0';
      else
        if axi = axi_IDLE then
          axi_writer_BREADY_sig <= '1';
        end if;
      end if;
    end if;
  end process;


  -- expressions
  tmp_0001 <= data_address_external_sig when ext_ctrl_sig = '1' else data_address_sig;
  tmp_0002 <= data_din_external_sig when ext_ctrl_sig = '1' else data_din_sig;
  tmp_0003 <= data_we_external_sig when ext_ctrl_sig = '1' else data_we_sig;
  tmp_0004 <= data_oe_external_sig when ext_ctrl_sig = '1' else data_oe_sig;
  tmp_0005 <= write_state_busy or read_state_busy;
  tmp_0006 <= tmp_0005 or forbid_sig;
  tmp_0007 <= not hdl_write_kick_d;
  tmp_0008 <= write_kick_sig and tmp_0007;
  tmp_0009 <= not tmp_0008;
  tmp_0010 <= not hdl_read_kick_d;
  tmp_0011 <= read_kick_sig and tmp_0010;
  tmp_0012 <= tmp_0009 and tmp_0011;
  tmp_0022 <= '1' when signed(tmp_0021) = 0 else '0';
  tmp_0023 <= std_logic_vector(signed(tmp_0021) - 1);
  tmp_0024 <= (others => '0') when tmp_0022 = '1' else tmp_0023;
  tmp_0026 <= BUF_address_b + 1;
  tmp_0027 <= tmp_0025 - 1;
  tmp_0029 <= tmp_0028 and axi_writer_WVALID_sig;
  tmp_0030 <= tmp_0029 and axi_writer_WREADY_sig;
  tmp_0031 <= std_logic_vector(signed(burst_size_sig) - 1);
  tmp_0032 <= tmp_0031(32 - 24 - 1 downto 0);
  tmp_0033 <= not axi_writer_AWREADY_sig;
  tmp_0034 <= tmp_0026 when axi_writer_AWREADY_sig = '1' else BUF_address_b;
  tmp_0035 <= '1' when signed(tmp_0021) > 0 else '0';
  tmp_0036 <= '1' when tmp_0035 = '1' else '0';
  tmp_0037 <= '1' when signed(tmp_0021) > 0 else '0';
  tmp_0038 <= '1' when tmp_0037 = '1' else '0';
  tmp_0039 <= '1' when signed(tmp_0021) > 0 else '0';
  tmp_0040 <= '1' when tmp_0039 = '1' else '0';
  tmp_0041 <= '1' when tmp_0025 = 1 else '0';
  tmp_0042 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0043 <= '0' when tmp_0042 = '1' else '1';
  tmp_0044 <= '1' when tmp_0025 = 2 else '0';
  tmp_0045 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0046 <= '1' when tmp_0045 = '1' else '0';
  tmp_0047 <= tmp_0046 when tmp_0044 = '1' else '0';
  tmp_0048 <= tmp_0043 when tmp_0041 = '1' else tmp_0047;
  tmp_0049 <= '1' when tmp_0025 > 1 else '0';
  tmp_0050 <= '1' when tmp_0025 = 1 else '0';
  tmp_0051 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0052 <= tmp_0050 and tmp_0051;
  tmp_0053 <= '0' when tmp_0052 = '1' else '1';
  tmp_0054 <= '1' when tmp_0049 = '1' else tmp_0053;
  tmp_0055 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0056 <= '1' when tmp_0025 > 0 else '0';
  tmp_0057 <= tmp_0055 and tmp_0056;
  tmp_0058 <= tmp_0027 when tmp_0057 = '1' else tmp_0025;
  tmp_0059 <= '1' when tmp_0025 = 1 else '0';
  tmp_0060 <= axi_writer_WVALID_sig and axi_writer_WREADY_sig;
  tmp_0061 <= tmp_0059 and tmp_0060;
  tmp_0064 <= std_logic_vector(signed(burst_size_sig) - 1);
  tmp_0065 <= tmp_0064(32 - 24 - 1 downto 0);
  tmp_0066 <= not axi_reader_ARREADY_sig;
  tmp_0067 <= '1' when tmp_0062 = 1 else '0';
  tmp_0068 <= tmp_0062 - 1;
  tmp_0069 <= tmp_0063 + 1;
  tmp_0070 <= tmp_0068 when axi_reader_RVALID_sig = '1' else tmp_0062;
  tmp_0071 <= tmp_0069 when axi_reader_RVALID_sig = '1' else tmp_0063;
  tmp_0072 <= axi_reader_RVALID_sig and tmp_0067;

  -- sequencers
  process (clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        axi <= axi_IDLE;
        axi_delay <= (others => '0');
      else
        case (axi) is
          when axi_IDLE => 
            if tmp_0008 = '1' then
              axi <= axi_sjr_tmp_state_0000;
            elsif tmp_0012 = '1' then
              axi <= axi_sjr_tmp_state_0005;
            end if;
          when axi_sjr_tmp_state_0000 => 
            if axi_writer_AWREADY_sig = '1' then
              axi <= axi_sjr_tmp_state_0001;
            end if;
          when axi_sjr_tmp_state_0001 => 
            axi <= axi_sjr_tmp_state_0002;
          when axi_sjr_tmp_state_0002 => 
            axi <= axi_sjr_tmp_state_0003;
          when axi_sjr_tmp_state_0003 => 
            if tmp_0061 = '1' then
              axi <= axi_sjr_tmp_state_0004;
            end if;
          when axi_sjr_tmp_state_0004 => 
            if axi_writer_BVALID_sig = '1' then
              axi <= axi_IDLE;
            end if;
          when axi_sjr_tmp_state_0005 => 
            if axi_reader_ARREADY_sig = '1' then
              axi <= axi_sjr_tmp_state_0006;
            end if;
          when axi_sjr_tmp_state_0006 => 
            if tmp_0072 = '1' then
              axi <= axi_IDLE;
            end if;
          when others => null;
        end case;
        hdl_write_kick_d <= write_kick_sig;
        hdl_read_kick_d <= read_kick_sig;
      end if;
    end if;
  end process;


  data_address_act <= signed(tmp_0001);

  data_din_act <= signed(tmp_0002);

  data_dout_act <= BUF_dout;

  data_we_act <= tmp_0003;

  data_oe_act <= tmp_0004;

  BUF_clk <= clk_sig;

  BUF_reset <= reset_sig;

  BUF_address <= data_address_act;

  BUF_din <= data_din_act;

  BUF_we <= data_we_act;

  BUF_oe <= data_oe_act;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        BUF_address_b <= (others => '0');
      else
        if axi = axi_sjr_tmp_state_0000 then
          BUF_address_b <= tmp_0034;
        elsif axi = axi_sjr_tmp_state_0001 then
          BUF_address_b <= tmp_0026;
        elsif axi = axi_sjr_tmp_state_0002 then
          BUF_address_b <= tmp_0026;
        elsif axi = axi_sjr_tmp_state_0003 then
          BUF_address_b <= tmp_0026;
        elsif axi = axi_IDLE then
          BUF_address_b <= (others => '0');
        elsif axi = axi_sjr_tmp_state_0006 then
          BUF_address_b <= tmp_0063;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        BUF_din_b <= (others => '0');
      else
        if axi = axi_sjr_tmp_state_0006 then
          BUF_din_b <= signed(axi_reader_RDATA_sig);
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        BUF_we_b <= '0';
      else
        if axi = axi_sjr_tmp_state_0006 then
          BUF_we_b <= axi_reader_RVALID_sig;
        else
          BUF_we_b <= '0';
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        BUF_oe_b <= '0';
      else
        if axi = axi_IDLE then
          BUF_oe_b <= tmp_0008;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        read_state_busy <= '0';
      else
        if axi = axi_IDLE then
          read_state_busy <= tmp_0012;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        write_state_busy <= '0';
      else
        if axi = axi_IDLE then
          write_state_busy <= tmp_0008;
        end if;
      end if;
    end if;
  end process;

  sjr_generate_U_0013_clk <= clk_sig;

  sjr_generate_U_0013_reset <= reset_sig;

  sjr_generate_U_0013_we <= tmp_0014;

  sjr_generate_U_0013_din <= tmp_0016;

  sjr_generate_U_0013_re <= tmp_0015;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        tmp_0014 <= '0';
      else
        if axi = axi_sjr_tmp_state_0001 then
          tmp_0014 <= tmp_0036;
        elsif axi = axi_sjr_tmp_state_0002 then
          tmp_0014 <= tmp_0038;
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0014 <= tmp_0040;
        else
          tmp_0014 <= '0';
        end if;
      end if;
    end if;
  end process;

  tmp_0015 <= tmp_0030;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        tmp_0016 <= (others => '0');
      else
        if axi = axi_sjr_tmp_state_0001 then
          tmp_0016 <= std_logic_vector(BUF_dout_b);
        elsif axi = axi_sjr_tmp_state_0002 then
          tmp_0016 <= std_logic_vector(BUF_dout_b);
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0016 <= std_logic_vector(BUF_dout_b);
        end if;
      end if;
    end if;
  end process;

  tmp_0017 <= sjr_generate_U_0013_dout;

  tmp_0018 <= sjr_generate_U_0013_empty;

  tmp_0019 <= sjr_generate_U_0013_empty;

  tmp_0020 <= sjr_generate_U_0013_count;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        tmp_0021 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0021 <= burst_size_sig;
        elsif axi = axi_sjr_tmp_state_0001 then
          tmp_0021 <= tmp_0024;
        elsif axi = axi_sjr_tmp_state_0002 then
          tmp_0021 <= tmp_0024;
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0021 <= tmp_0024;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        tmp_0025 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0025 <= signed(burst_size_sig);
        elsif axi = axi_sjr_tmp_state_0003 then
          tmp_0025 <= tmp_0058;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        tmp_0028 <= '0';
      else
        if axi = axi_sjr_tmp_state_0003 then
          tmp_0028 <= '1';
        else
          tmp_0028 <= '0';
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        tmp_0062 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0062 <= signed(burst_size_sig);
        elsif axi = axi_sjr_tmp_state_0006 then
          tmp_0062 <= tmp_0070;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        tmp_0063 <= (others => '0');
      else
        if axi = axi_IDLE then
          tmp_0063 <= (others => '0');
        elsif axi = axi_sjr_tmp_state_0006 then
          tmp_0063 <= tmp_0071;
        end if;
      end if;
    end if;
  end process;


  inst_BUF : dualportram
  generic map(
    WIDTH => 32,
    DEPTH => 10,
    WORDS => 1024
  )
  port map(
    clk => BUF_clk,
    reset => BUF_reset,
    length => BUF_length,
    address => BUF_address,
    din => BUF_din,
    dout => BUF_dout,
    we => BUF_we,
    oe => BUF_oe,
    address_b => BUF_address_b,
    din_b => BUF_din_b,
    dout_b => BUF_dout_b,
    we_b => BUF_we_b,
    oe_b => BUF_oe_b
  );

  inst_sjr_generate_U_0013 : simple_fifo_32
  port map(
    clk => sjr_generate_U_0013_clk,
    reset => sjr_generate_U_0013_reset,
    we => sjr_generate_U_0013_we,
    din => sjr_generate_U_0013_din,
    re => sjr_generate_U_0013_re,
    dout => sjr_generate_U_0013_dout,
    empty => sjr_generate_U_0013_empty,
    full => sjr_generate_U_0013_full,
    count => sjr_generate_U_0013_count
  );


end RTL;
